unit Main;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs, FMX.Platform.Win,
  BeeperWrapper, WinRing0, FMX.StdCtrls, FMX.Controls.Presentation, Windows,
  FMX.Objects;

type
  TMainForm = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure GenerateKeyboard(Width, Height, OctavesCount: Integer; const Parent: TFMXObject);
    procedure FillImage(const Image: TImage; Color: TAlphaColor; DrawBorder: Boolean = True);
    procedure RepaintKeys;
    procedure OnKeyPaint(Sender: TObject; Canvas: TCanvas; const ARect: TRectF);
    procedure OnKeyEnter(Sender: TObject);
    procedure OnKeyLeave(Sender: TObject);
    procedure OnKeyDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Single);
    procedure OnKeyUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Single);
    procedure FormDestroy(Sender: TObject);
  private
    FIsMouseDown: Boolean;
    FKeyPressedTag: Integer;
    FKeyHoveredTag: Integer;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.fmx}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

function GetFmxWND(const WindowHandle: TWindowHandle): THandle; inline;
begin
  Result := WindowHandleToPlatform(WindowHandle).Wnd
end;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

procedure ShowErrorMessage(Text: string);
begin
  MessageBox(GetFmxWND(MainForm.Handle), PChar(Text), '!', MB_ICONERROR);
end;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

procedure TMainForm.FillImage(const Image: TImage; Color: TAlphaColor;
  DrawBorder: Boolean = True);
begin
  Image.Canvas.Fill.Color := Color;

  //    :
  Image.Canvas.FillRect(
                         TRectF.Create(
                                        TPointF.Create(0, 0),
                                        TPointF.Create(Image.Width, Image.Height)
                                       ),
                         0.5,
                         0.5,
                         AllCorners,
                         1,
                         TCornerType.InnerLine
                        );

  //  :
  if DrawBorder then
    Image.Canvas.DrawRect(
                           TRectF.Create(
                                          TPointF.Create(0, 0),
                                          TPointF.Create(Image.Width, Image.Height)
                                         ),
                           0.5,
                           0.5,
                           AllCorners,
                           1,
                           TCornerType.InnerLine
                          );
end;

//HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH

procedure TMainForm.FormCreate(Sender: TObject);
var
  Status: LongWord;
begin
  Status := InitializeBeeper;
  case Status of
    OLS_DLL_UNSUPPORTED_PLATFORM         : ShowErrorMessage(' !');
    OLS_DLL_DRIVER_NOT_LOADED            : ShowErrorMessage('   !');
    OLS_DLL_DRIVER_NOT_FOUND             : ShowErrorMessage('  !');
    OLS_DLL_DRIVER_UNLOADED              : ShowErrorMessage('   !');
    OLS_DLL_DRIVER_NOT_LOADED_ON_NETWORK : ShowErrorMessage('     !');
    OLS_DLL_UNKNOWN_ERROR                : ShowErrorMessage(' !');
  end;

  if Status <> OLS_DLL_NO_ERROR then ExitProcess(0);

  FIsMouseDown := False;
  FKeyPressedTag := -1;
  FKeyHoveredTag := -1;

  MainForm.BeginUpdate;
  GenerateKeyboard(ClientWidth, ClientHeight, 5, MainForm);
  MainForm.EndUpdate;

  //     :
  SetBeeperRegime;
  SetBeeperDivider(0);
end;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  StopBeeper;
end;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

procedure TMainForm.GenerateKeyboard(Width, Height, OctavesCount: Integer; const Parent: TFMXObject);
var
  I: Integer;
  Image: TImage;
  KeysCount: Integer;
  KeyWidth: Single;
  InternalCounter: Integer;
  WhiteCounter: Integer;
const
  CornerRadius = 1.0;
  CornerOpacity = 1.0;
begin
  if (OctavesCount <= 0) or (Width <= 0) or (Height <= 0) then Exit;

  KeysCount := OctavesCount * 12;
  KeyWidth := Width / (OctavesCount * 7);

  WhiteCounter := 0;
  for I := 0 to KeysCount - 1 do
  begin
    Image := TImage.Create(MainForm);
    Image.Parent := MainForm;
    Image.Tag := I;
    Image.Visible := True;
    Image.OnPaint := OnKeyPaint;
    Image.OnMouseEnter := OnKeyEnter;
    Image.OnMouseLeave := OnKeyLeave;
    Image.OnMouseDown  := OnKeyDown;
    Image.OnMouseUp    := OnKeyUp;
    Image.HitTest := True;

    InternalCounter := (I - ((I div 12) * 12)) + 1;
    case InternalCounter of
      2, 4, 7, 9, 11:
      begin
        Image.Position.X := (WhiteCounter * KeyWidth) - (0.25 * KeyWidth);
        Image.Position.Y := 0;
        Image.Width  := 0.5 * KeyWidth;
        Image.Height := 0.5 * Height;
        Image.BringToFront;
      end;

    else
      Image.Position.X := WhiteCounter * KeyWidth;
      Image.Position.Y := 0;
      Image.Width  := KeyWidth;
      Image.Height := Height;
      Image.SendToBack;
      Inc(WhiteCounter);
    end;
  end;
end;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

procedure TMainForm.OnKeyPaint(Sender: TObject; Canvas: TCanvas;
  const ARect: TRectF);
var
  Image: TImage;
  InternalCounter: Integer;
begin
  Image := TImage(Sender);

  InternalCounter := (Image.Tag - ((Image.Tag div 12) * 12)) + 1;
  if (Image.Tag <> FKeyPressedTag) and (Image.Tag <> FKeyHoveredTag) then
  begin
    case InternalCounter of
      2, 4, 7, 9, 11: FillImage(Image, $FF000000);
    else
      FillImage(Image, $FFFFFFFF);
    end;
  end
  else
  begin
    if Image.Tag = FKeyPressedTag then
      FillImage(Image, $FF0060FF)
    else if Image.Tag = FKeyHoveredTag then
      FillImage(Image, $FF00FF00);
  end;
end;

//HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH

procedure TMainForm.RepaintKeys;
var
  Component: TComponent;
begin
  MainForm.BeginUpdate;
  for Component in MainForm do
    if Component is TImage then
      TImage(Component).Repaint;
  MainForm.EndUpdate;
end;

//HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH

procedure TMainForm.OnKeyDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Single);
begin
  FIsMouseDown := True;
  FKeyPressedTag := TImage(Sender).Tag;
  SetBeeperFrequency(ChromaticGamma[TImage(Sender).Tag]);
  StartBeeper;
  RepaintKeys;
end;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

procedure TMainForm.OnKeyUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Single);
begin
  FIsMouseDown := False;
  StopBeeper;
  FKeyPressedTag := -1;
  RepaintKeys;
end;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

procedure TMainForm.OnKeyEnter(Sender: TObject);
begin
  FKeyHoveredTag := TImage(Sender).Tag;

  if FIsMouseDown then
  begin
    FKeyPressedTag := TImage(Sender).Tag;
    SetBeeperFrequency(ChromaticGamma[FKeyPressedTag]);
  end
  else
  begin
    StopBeeper;
  end;

  RepaintKeys;
end;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

procedure TMainForm.OnKeyLeave(Sender: TObject);
begin
  if not FIsMouseDown then StopBeeper;
  if FKeyHoveredTag = TImage(Sender).Tag then FKeyHoveredTag := -1;
  RepaintKeys;
end;

end.
